@@ -1,14 +1,20 @@
+#  You may distribute under the terms of either the GNU General Public License
+#  or the Artistic License (the same terms as Perl itself)
+#
+#  (C) Paul Evans, 2014 -- leonerd@leonerd.org.uk
+
 package Device::BusPirate;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 use Carp;
 
 use Future::Utils qw( repeat );
 use IO::Termios;
+use Time::HiRes qw( time );
 
 use Module::Pluggable
    search_path => "Device::BusPirate::Mode",
@@ -16,6 +22,14 @@ use Module::Pluggable
    sub_name    => "modes";
 my %MODEMAP = map { $_->MODE => $_ } __PACKAGE__->modes;
 
+use Module::Pluggable
+   search_path => "Device::BusPirate::Chip",
+   require     => 1,
+   sub_name    => "chips";
+my %CHIPMAP = map { $_->CHIP => $_ } __PACKAGE__->chips;
+
+use constant PIRATE_DEBUG => $ENV{PIRATE_DEBUG};
+
 =head1 NAME
 
 C<Device::BusPirate> - interact with a F<Bus Pirate> device
@@ -37,10 +51,26 @@ This module and its various component modules are based on L<Future>, allowing
 either synchronous or asynchronous communication with the attached hardware
 device. For simple synchronous situations, the class may be used on its own,
 and any method that returns a C<Future> instance should immediately call the
-C<get> method on that instance to wait for and obtain the eventual result.
+C<get> method on that instance to wait for and obtain the eventual result:
 
  my $spi = $pirate->enter_mode( "SPI" )->get;
 
+ $spi->power( 1 )->get;
+ my $input = $spi->writeread_cs( $output )->get;
+
+A truely-asynchronous program would be built using a subclass that overrides
+the basic C<read> and C<sleep> methods for some event loop or similar; these
+can then be chained using the C<then> method:
+
+ my $input = $pirate->enter_mode( "SPI" )
+   ->then( sub {
+      my ( $spi ) = @_;
+
+      $spi->power( 1 )->then( sub {
+         $spi->writeread_cs( $output );
+      });
+   });
+
 =cut
 
 =head1 CONSTRUCTOR
@@ -102,6 +132,8 @@ sub write
    my $self = shift;
    my ( $buf ) = @_;
 
+   printf STDERR "PIRATE >> %v02x\n", $buf if PIRATE_DEBUG;
+
    $self->{fh}->syswrite( $buf );
 }
 
@@ -122,13 +154,35 @@ sub sleep
    my $self = shift;
    my ( $timeout ) = @_;
 
-   croak "Cannot sleep twice" if $self->{sleep_f};
-   $self->{sleep_f} = [ $timeout, my $f = $self->_new_future ];
-   $f->on_cancel( sub { undef $self->{sleep_f} } );
+   my $until = time() + $timeout;
+
+   croak "Cannot sleep less than existing timeout" if
+      $self->{alarms}[0] and $until < $self->{alarms}[0][0];
+
+   my $alarm = push @{ $self->{alarms} }, [ $until, my $f = $self->_new_future ];
+   $f->on_cancel( sub {
+      $self->{alarms} = [ grep { $_ != $alarm } @{ $self->{alarms} } ]
+   });
 
    return $f;
 }
 
+sub _enter_txn
+{
+   my $self = shift;
+   my ( $code ) = @_;
+
+   my $oldlock = $self->{txn_lock} // $self->_new_future->done( $self );
+   $self->{txn_lock} = my $newlock = $self->_new_future;
+
+   $oldlock->then( $code )
+      ->then_with_f( sub {
+         my $f = shift;
+         $newlock->done( $self );
+         $f
+      });
+}
+
 =head2 $mode = $pirate->enter_mode( $modename )->get
 
 Switches the attached device into the given mode, and returns an object to
@@ -167,6 +221,36 @@ sub enter_mode
    });
 }
 
+=head2 $chip = $pirate->mount_chip( $chipname, %opts )->get
+
+Constructs a "chip" object; a helper designed to communicate with some
+particular hardware device (usually a specific chip) attached to the Bus
+Pirate. This will be a subclass of L<Device::BusPirate::Chip>, and will likely
+provide various methods specific to the operation of that particular device.
+
+C<$chipname> should match the name declared by the chip module, and other
+options passed in C<%opts> will be passed to its constructor.
+
+=cut
+
+sub mount_chip
+{
+   my $self = shift;
+   my ( $chipname, %opts ) = @_;
+
+   my $chipclass = $CHIPMAP{$chipname} or
+      croak "Unrecognised chip '$chipname'";
+
+   my $chip = $self->{chip} = $chipclass->new( $self, %opts );
+
+   $self->enter_mode( $chip->MODE )
+      ->then( sub {
+         my ( $mode ) = @_;
+         $chip->mount( $mode )
+      })
+      ->then_done( $chip );
+}
+
 =head2 $pirate->start->get
 
 Starts binary IO mode on the F<Bus Pirate> device, enabling the module to
@@ -220,6 +304,8 @@ package Device::BusPirate::_Future;
 use base qw( Future );
 use Carp;
 
+use Time::HiRes qw( time );
+
 sub new
 {
    my $proto = shift;
@@ -232,30 +318,32 @@ sub await
 {
    my $bp = shift->{bp};
 
-   my $sleep = $bp->{sleep_f};
+   my $alarm = $bp->{alarms}[0];
    my $read = $bp->{read_f}[0];
 
    my $fh = $bp->{fh};
 
-   croak "Cannot await with nothing to do" unless $sleep or $read;
+   croak "Cannot await with nothing to do" unless $alarm or $read;
 
    my $buf = '';
 
    while( length $buf < $read->[0] ) {
-      my $timeout = $sleep ? $sleep->[0] : undef;
+      my $timeout = $alarm ? $alarm->[0] - time() : undef;
       my $rvec = '';
       vec( $rvec, $fh->fileno, 1 ) = 1 if $read;
 
       if( select( $rvec, undef, undef, $timeout ) ) {
          $fh->sysread( $buf, $read->[0] - length $buf, length $buf );
       }
-      elsif( $sleep ) {
-         undef $bp->{sleep_f};
-         $sleep->[1]->done;
+      elsif( $timeout ) {
+         shift @{ $bp->{alarms} };
+         $alarm->[1]->done;
          return;
       }
    }
 
+   printf STDERR "PIRATE << %v02x\n", $buf if Device::BusPirate::PIRATE_DEBUG;
+
    shift @{ $bp->{read_f} };
    $read->[1]->done( $buf );
 }
@@ -270,8 +358,7 @@ More modes - I2C, UART, 1-wire, raw-wire
 
 =item *
 
-Concept of "chips" - models of attached devices and defined ways to interact
-with them. Maybe "chip" isn't the best name for e.g. MIDI over UART or PS/2.
+Documentation/examples/actual implementations of truely-async subclass.
 
 =item *
 
